<div id="chatwindow"><span id="loading">Loading...</span></div>

<form id="chatform">
	<label for="msg">Message:</label>
	<input type="text" id="msg" />
	<input type="submit" value="Send message" /><br />
</form>

